@@ -9,7 +9,8 @@ import sys
 import time
 import operator
 from . import pv
-   
+from .utils import is_string
+
 class Alarm(object):
     """ alarm class for a PV:
     run a user-supplied callback when a PV's value goes out of range
@@ -87,7 +88,7 @@ class Alarm(object):
 
         if isinstance(pvname, pv.PV):
             self.pv = pvname
-        elif isinstance(pvname, str):
+        elif is_string(pvname):
             self.pv = pv.PV(pvname)
             self.pv.connect()
         
