@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #  M Newville <newville@cars.uchicago.edu>
 #  The University of Chicago, 2010
 #  Epics Open License
@@ -235,12 +235,11 @@ class Device(object):
         elif attr in self.__dict__:
             return self.__dict__[attr]
         elif self._init and not attr.startswith('__'):
-            try:
-                self.PV(attr)
-                return self.get(attr)
-            except:
-                msg = "Device '%s' has no attribute '%s'"
+            pv = self.PV(attr, connect=True)
+            if not pv.connected:
+                msg = "%s '%s' has no attribute '%s'" % self.__class__.__name__
                 raise AttributeError(msg % (self._prefix, attr))
+            return pv.get()
 
     def __setattr__(self, attr, val):
         if attr in self._nonpvs:
