@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 #  M Newville <newville@cars.uchicago.edu>
 #  The University of Chicago, 2010
 #  Epics Open License
@@ -12,14 +12,16 @@ from math import log10
 
 from . import ca
 from . import dbr
+from .utils import is_string
 
 def fmt_time(tstamp=None):
     "simple formatter for time values"
     if tstamp is None:
         tstamp = time.time()
     tstamp, frac = divmod(tstamp, 1)
-    return "%s.%6.6i" % (time.strftime("%Y-%m-%d %H:%M:%S",
-                                       time.localtime(tstamp)), 1.e6*frac)
+    return "%s.%5.5i" % (time.strftime("%Y-%m-%d %H:%M:%S",
+                                       time.localtime(tstamp)),
+                         round(1.e5*frac))
 
 class PV(object):
     """Epics Process Variable
@@ -257,7 +259,7 @@ class PV(object):
             return None
 
         if (self.ftype in (dbr.ENUM, dbr.TIME_ENUM, dbr.CTRL_ENUM) and
-            isinstance(value, str)):
+            is_string(value)):
             if self._args['enum_strs'] is None:
                 self.get_ctrlvars()
             if value in self._args['enum_strs']:
@@ -289,58 +291,61 @@ class PV(object):
         if ntype == dbr.STRING:
             self._args['char_value'] = val
             return val
+        # char waveform as string
+        if ntype == dbr.CHAR and self.count < ca.AUTOMONITOR_MAXLENGTH:
+            if self.count==1: # handles single character in waveform
+                val = [val]
+            val = list(val)
+            if 0 in val:
+                firstnull  = val.index(0)
+            else:
+                firstnull = len(val)
+            try:
+                cval = ''.join([chr(i) for i in val[:firstnull]]).rstrip()
+            except ValueError:
+                cval = ''
+            self._args['char_value'] = cval
+            return cval
+        
         cval  = repr(val)
         if self.count > 1:
-            if ntype == dbr.CHAR and self.count < ca.AUTOMONITOR_MAXLENGTH:
-                val = list(val)
-                if 0 in val:
-                    firstnull  = val.index(0)
-                else:
-                    firstnull = len(val)
-                try:
-                    cval = ''.join([chr(i) for i in val[:firstnull]]).rstrip()
-                except ValueError:
-                    pass
-            else:
-                cval = '<array size=%d, type=%s>' % (len(val),
-                                                     dbr.Name(ftype).lower())
+            cval = '<array size=%d, type=%s>' % (len(val),
+                                                 dbr.Name(ftype).lower())
         elif ntype in (dbr.FLOAT, dbr.DOUBLE):
             if call_ca and self._args['precision'] is None:
                 self.get_ctrlvars()
             try:
-                prec = getattr(self, 'precision')
+                prec = self._args['precision']
                 fmt  = "%%.%if"
                 if 4 < abs(int(log10(abs(val + 1.e-9)))):
                     fmt = "%%.%ig"
                 cval = (fmt %  prec) % val
             except (ValueError, TypeError, ArithmeticError):
-                self._args['char_value'] = str(val)
-                return self._args['char_value']
-
+                cval = str(val)
         elif ntype == dbr.ENUM:
             if call_ca and self._args['enum_strs'] in ([], None):
                 self.get_ctrlvars()
             try:
                 cval = self._args['enum_strs'][val]
             except (TypeError, KeyError,  IndexError):
-                pass
+                cval = str(val)
+
         self._args['char_value'] = cval
         return cval
 
-    def get_ctrlvars(self):
+    def get_ctrlvars(self, timeout=5, warn=True):
         "get control values for variable"
-        return self._get_vars(ca.get_ctrlvars)
-
-    def get_timevars(self):
-        "get time values for variable"
-        return self._get_vars(ca.get_timevars)
-
-    def _get_vars(self, var_fn):
-        "internal, common functionality for retreiving control/times values"
         if not self.wait_for_connection():
             return None
-        kwds = var_fn(self.chid)
-        ca.poll()
+        kwds = ca.get_ctrlvars(self.chid, timeout=timeout, warn=warn)
+        self._args.update(kwds)
+        return kwds
+        
+    def get_timevars(self, timeout=5, warn=True):
+        "get time values for variable"
+        if not self.wait_for_connection():
+            return None
+        kwds = ca.get_timevars(self.chid, timeout=timeout, warn=warn)
         self._args.update(kwds)
         return kwds
 
@@ -352,7 +357,7 @@ class PV(object):
         """
         self._args.update(kwd)
         self._args['value']  = value
-        self._args['timestamp'] = kwd.get('timestamp', time.time())
+        self._args['timestamp'] = kwd.get('timestamp', None)
         self._set_charval(self._args['value'], call_ca=False)
         if self.verbose:
             now = fmt_time(self._args['timestamp'])
@@ -408,10 +413,9 @@ class PV(object):
         if with_ctrlvars and self.connected:
             self.get_ctrlvars()
         if run_now:
-            if not self.connected:
-                self.wait_for_connection()
-        if run_now and self.connected:
-            self.run_callback(index)
+            self.get(as_string=True)
+            if self.connected:
+                self.run_callback(index)
         return index
 
     def remove_callback(self, index=None):
@@ -495,6 +499,11 @@ class PV(object):
         "wrapper for property retrieval"
         if self._args['value'] is None:
             self.get()
+        if self._args[arg] is None:
+            if arg in ('status', 'severity', 'timestamp'):
+                self.get_timevars(timeout=1, warn=False)
+            else:
+                self.get_ctrlvars(timeout=1, warn=False)
         return self._args.get(arg, None)
 
     def __getval__(self):
