@@ -105,6 +105,56 @@ class PVMixin(object):
         if pv is not None:
             self.SetPV(pv)
 
+        self._popupmenu = None
+        self.Bind(wx.EVT_RIGHT_DOWN, self._onRightDown)
+            
+    def _onRightDown(self, event=None):
+        """right button down: show pop-up"""
+        if event is None:
+            return
+        if self._popupmenu is None:
+            self.build_popupmenu()
+        wx.CallAfter(self.PopupMenu, self._popupmenu, event.GetPosition())
+        event.Skip()
+    
+    def build_popupmenu(self, event=None):
+        self._copy_name = wx.NewId()
+        self._copy_val  = wx.NewId()
+        self._show_info = wx.NewId()
+        self._popupmenu = wx.Menu()
+        self._popupmenu.Append(self._copy_name, 'Copy PV Name to Clipboard')
+        self._popupmenu.Append(self._copy_val,  'Copy PV Value to Clipboard')
+        self._popupmenu.AppendSeparator()
+        self._popupmenu.Append(self._show_info, 'Show PV Info')
+
+        self.Bind(wx.EVT_MENU, self.copy_name,  id=self._copy_name)
+        self.Bind(wx.EVT_MENU, self.copy_val,   id=self._copy_val)
+        self.Bind(wx.EVT_MENU, self.show_info,  id=self._show_info)
+        
+    @EpicsFunction
+    def copy_name(self, event=None):
+        dat = wx.TextDataObject()
+        dat.SetText(self.pv.pvname)
+        wx.TheClipboard.Open()
+        wx.TheClipboard.SetData(dat)
+        wx.TheClipboard.Close()
+        
+    @EpicsFunction
+    def copy_val(self, event=None):
+        dat = wx.TextDataObject()
+        dat.SetText(self.pv.get(as_string=True))
+        wx.TheClipboard.Open()
+        wx.TheClipboard.SetData(dat)
+        wx.TheClipboard.Close()
+
+    @EpicsFunction
+    def show_info(self, event=None):
+        dlg = wx.MessageDialog(self, self.pv.info,
+                               'Info for %s' % self.pv.pvname,
+                               wx.OK|wx.ICON_INFORMATION)
+        dlg.ShowModal()
+        dlg.Destroy()
+
     @EpicsFunction
     def SetPV(self, pv=None):
         "set pv, either an epics.PV object or a pvname"
@@ -113,11 +163,14 @@ class PVMixin(object):
         if isinstance(pv, epics.PV):
             self.pv = pv
         elif isinstance(pv, (str, unicode)):
-            self.pv = epics.PV(pv)
+            form = "ctrl" if len(self._fg_colour_alarms) > 0 or len(self._bg_colour_alarms) > 0 else "native"
+            self.pv = epics.PV(pv, form=form)
             self.pv.connect()
 
         epics.poll()
         self.pv.connection_callbacks.append(self.OnEpicsConnect)
+        if self.pv.connected:
+            self.OnEpicsConnect(pvname=self.pv.pvname, conn=True, pv=self.pv)
 
         self.pv.get_ctrlvars()
 
@@ -389,7 +442,10 @@ class PVCtrlMixin(PVMixin):
             elif self.pv.severity in self._bg_colour_alarms:
                 colour = self._bg_colour_alarms[self.pv.severity]
             self.OverrideBackgroundColour(colour)
-            self._SetValue(self._translations.get(raw_value, raw_value))
+            try:
+                self._SetValue(self._translations.get(raw_value, raw_value))
+            except TypeError:
+                pass
         except PyDeadObjectError:
             pass
 
@@ -401,7 +457,7 @@ class PVTextCtrl(wx.TextCtrl, PVCtrlMixin):
     back on input
     """
     def __init__(self, parent,  pv=None,
-                 font=None, fg=None, bg=None, **kws):
+                 font=None, fg=None, bg=None, dirty_timeout=2500, **kws):
 
         if 'style' not in kws:
             kws['style'] = wx.TE_PROCESS_ENTER
@@ -411,16 +467,12 @@ class PVTextCtrl(wx.TextCtrl, PVCtrlMixin):
         wx.TextCtrl.__init__(self, parent, wx.ID_ANY, value='', **kws)
         PVCtrlMixin.__init__(self, pv=pv, font=font, fg=fg, bg=bg)
         self.Bind(wx.EVT_CHAR, self.OnChar)
+        self.dirty_timeout = dirty_timeout
+        self.dirty_writeback_timer = wx.Timer(self)
+        self.Bind(wx.EVT_TIMER, self.OnWriteback)
+        self.Bind(wx.EVT_KILL_FOCUS, self.OnWriteback)
+        self.Bind(wx.EVT_CHAR, self.OnChar)
 
-    def OnChar(self, event):
-        "char event handler"
-        key   = event.GetKeyCode()
-        entry = str(wx.TextCtrl.GetValue(self).strip())
-        pos   = wx.TextCtrl.GetSelection(self)
-        if key == wx.WXK_RETURN:
-            self.SetValue(entry)
-        else:
-            event.Skip()
 
     @EpicsFunction
     def _caput(self, value):
@@ -435,6 +487,21 @@ class PVTextCtrl(wx.TextCtrl, PVCtrlMixin):
         "set widget value"
         wx.TextCtrl.SetValue(self, value)
 
+    def OnChar(self, event):
+        "char event handler"
+        if event.KeyCode == wx.WXK_RETURN:
+            self.OnWriteback()
+        else:
+            if self.dirty_timeout is not None:
+                self.dirty_writeback_timer.Start(self.dirty_timeout)
+            event.Skip()
+
+    def OnWriteback(self, event=None):
+        """ writeback the currently displayed value to the PV """
+        self.dirty_writeback_timer.Stop()
+        entry = str(self.GetValue().strip())
+        self.SetValue(entry)
+
 class PVText(wx.StaticText, PVCtrlMixin):
     """ Static text for displaying a PV value,
         with callback for automatic updates
@@ -522,6 +589,7 @@ class PVEnumButtons(wx.Panel, PVCtrlMixin):
                 b = buttons.GenToggleButton(self, -1, label)
                 self.buttons.append(b)
                 b.Bind(wx.EVT_BUTTON, Closure(self._onButton, index=i) )
+                b.Bind(wx.EVT_RIGHT_DOWN, self._onRightDown)
                 sizer.Add(b, flag = wx.ALL)
                 b.SetToggle(label==pv_value)
 
@@ -530,7 +598,6 @@ class PVEnumButtons(wx.Panel, PVCtrlMixin):
         sizer.Fit(self)
 
 
-
     @EpicsFunction
     def _onButton(self, event=None, index=None, **kw):
         "button event handler"
@@ -958,6 +1025,22 @@ class PVComboBox(wx.ComboBox, PVCtrlMixin):
         "text event"
         self.pv.put(self.Value)
 
+class PVEnumComboBox(PVComboBox):
+    """ A dropdown ComboBox linked to an enum PV, allows setting of enum values.
+    Both reads/writes the combo value on changes
+    """
+    def __init__(self, parent, pv=None, **kw):
+        PVComboBox.__init__(self, parent, pv, style=wx.CB_DROPDOWN|wx.CB_READONLY, **kw)
+
+    @DelayedEpicsCallback
+    def OnEpicsConnect(self, pvname=None, conn=None, pv=None):
+        PVComboBox.OnEpicsConnect(self, pvname, conn, pv)
+        if conn:
+            self.Clear()
+            for enum in self.pv.enum_strs:
+                self.Append(enum)
+            self.Value = self.pv.get(as_string=True)
+
 class PVToggleButton(wx.ToggleButton, PVCtrlMixin):
     """A ToggleButton that can be attached to a bi or bo Epics record."""
 
