@@ -45,6 +45,12 @@ def resume_updating():
     caput(pvnames.pause_pv, 0)
 
 class CA_BasicTests(unittest.TestCase):
+
+    def setUp(self):
+        write('Starting %s...' % self.id().split(".")[-1])
+    def tearDown(self):
+        write('Completed %s...\n' % self.id().split(".")[-1])
+
     def testA_CreateChid(self):
         write('Simple Test: create chid')
         chid = ca.create_channel(pvnames.double_pv)
@@ -146,6 +152,17 @@ class CA_BasicTests(unittest.TestCase):
         self.assertEqual(conn,False)
 
 
+    def test_putwait(self):
+        'test put with wait'
+        pvn = pvnames.non_updating_pv
+        chid = ca.create_channel(pvn, connect=True)
+        o  = ca.put(chid, -1, wait=True)
+        val = ca.get(chid)
+        self.assertEqual(val, -1)
+        o  = ca.put(chid, 2, wait=True)
+        val = ca.get(chid)
+        self.assertEqual(val, 2)
+
     def test_promote_type(self):
         pvn = pvnames.double_pv
         chid = ca.create_channel(pvn,connect=True)
@@ -223,6 +240,56 @@ class CA_BasicTests(unittest.TestCase):
         self.assertNotEqual(val, None)
         time.sleep(0.2)
 
+
+    def _test_array_callback(self, arrayname, array_type, length, element_type):
+        """ Helper function to subscribe to a PV array and check it
+        receives at least one subscription callback w/ specified type,
+        length & uniform element type. Checks separately for normal,
+        TIME & CTRL subscription variants. Returns the array or fails
+        an assertion."""
+        results = {}
+        for form in [ 'normal', 'time', 'ctrl' ]:
+            chid = ca.create_channel(arrayname,connect=True)
+            cb, uarg, eventID = ca.create_subscription(chid, use_time=form=='time', use_ctrl=form=='ctrl', callback=onChanges)
+
+            CHANGE_DAT.pop(arrayname, None)
+            timeout=0
+            # wait up to 6 seconds, if no callback probably due to simulator.py
+            # not running...
+            while timeout<120 and not arrayname in CHANGE_DAT:
+                time.sleep(0.05)
+                timeout = timeout+1
+            val = CHANGE_DAT.get(arrayname, None)
+            ca.clear_subscription(eventID)
+            self.assertNotEqual(val, None)
+            self.assertEqual(type(val), array_type)
+            self.assertEqual(len(val), length)
+            self.assertEqual(type(val[0]), element_type)
+            self.assertTrue(all( type(e)==element_type for e in val))
+            results[form] = val
+        return results
+
+    def test_subscription_long_array(self):
+        """ Check that numeric arrays callbacks successfully send correct data """
+        self._test_array_callback(pvnames.long_arr_pv, numpy.ndarray, 2048, numpy.int32)
+
+    def test_subscription_double_array(self):
+        """ Check that double arrays callbacks successfully send correct data """
+        self._test_array_callback(pvnames.double_arr_pv, numpy.ndarray, 2048, numpy.float64)
+
+    def test_subscription_string_array(self):
+        """ Check that string array callbacks successfully send correct data """
+        results = self._test_array_callback(pvnames.string_arr_pv, list, 128, str)
+        self.assertTrue(len(results["normal"][0]) > 0)
+        self.assertTrue(len(results["time"][0]) > 0)
+        self.assertTrue(len(results["ctrl"][0]) > 0)
+
+    def test_subscription_char_array(self):
+        """ Check that uchar array callbacks successfully send correct data as arrays """
+        self._test_array_callback(pvnames.char_arr_pv, numpy.ndarray, 128, numpy.uint8)
+
+
+
     def test_Values(self):
         write( 'CA test Values (compare 5 values with caget)')
         os.system('rm ./caget.tst')
@@ -327,7 +394,7 @@ class CA_BasicTests(unittest.TestCase):
         self.assertTrue(isinstance(lval, list))
         self.assertTrue(len(lval) > 2)
         self.assertTrue(lval == list(aval))
-        
+
     def test_xArray1(self):
         write('Array Test: get(wait=False) / get_complete()')
         chid = ca.create_channel(pvnames.double_arrays[0])
@@ -347,7 +414,7 @@ class CA_BasicTests(unittest.TestCase):
 
         out = ca.get(chid, count=0)
         self.assertTrue(isinstance(out, numpy.ndarray))
-        self.assertTrue(len(out) == npts)
+        self.assertEqual(len(out), npts)
 
     def test_xArray3(self):
         write('Array Test: get char array as string')
@@ -357,7 +424,7 @@ class CA_BasicTests(unittest.TestCase):
         char_val = ca.get(chid, as_string=True)
         self.assertTrue(isinstance(char_val, str))
         conv = ''.join([chr(i) for i in val])
-        self.assertTrue(conv == char_val)
+        self.assertEqual(conv, char_val)
 
 
 if __name__ == '__main__':
