@@ -6,7 +6,7 @@ import sys
 import time
 import unittest
 import numpy
-from epics import PV, caput
+from epics import PV, caput, caget, ca
 
 import pvnames
 
@@ -21,7 +21,7 @@ def onConnect(pvname=None, conn=None, chid=None,  **kws):
     write('  :Connection status changed:  %s  connected=%s\n' % (pvname, conn))
     global CONN_DAT
     CONN_DAT[pvname] = conn
-    
+
 def onChanges(pvname=None, value=None, **kws):
     write( '/// New Value: %s  value=%s, kw=%s\n' %( pvname, str(value), repr(kws)))
     global CHANGE_DAT
@@ -44,11 +44,20 @@ class PV_Tests(unittest.TestCase):
         pv = PV(pvnames.int_pv,
                 connection_callback=onConnect)
         val = pv.get()
-        
+
         global CONN_DAT
         conn = CONN_DAT.get(pvnames.int_pv, None)
         self.assertEqual(conn, True)
-        
+
+    def test_caget(self):
+        write('Simple Test of caget() function\n')
+        pvs = (pvnames.double_pv, pvnames.enum_pv, pvnames.str_pv)
+        for p in pvs:
+            val = caget(p)
+            self.assertNotEqual(val, None)
+        sval = caget(pvnames.str_pv)
+        self.assertEqual(sval, 'ao')
+
     def test_get1(self):
         write('Simple Test: test value and char_value on an integer\n')
         pause_updating()
@@ -59,7 +68,7 @@ class PV_Tests(unittest.TestCase):
         self.failUnless(int(cval)== val)
         resume_updating()
 
-        
+
     def test_stringarray(self):
         write('String Array: \n')
         pause_updating()
@@ -72,6 +81,24 @@ class PV_Tests(unittest.TestCase):
         self.failUnless(len(val[1]) > 1)
         resume_updating()
 
+    def test_putcomplete(self):
+        write('Put with wait and put_complete (using real motor!) \n')
+        vals = (1.35, 1.50, 1.44, 1.445, 1.45, 1.453, 1.446, 1.447, 1.450, 1.450, 1.490, 1.5, 1.500)
+        p = PV(pvnames.motor1)
+        see_complete = []
+        for v in vals:
+            t0 = time.time()
+            p.put(v, use_complete=True)
+            count = 0
+            for i in range(100000):
+                time.sleep(0.001)
+                count = count + 1
+                if p.put_complete:
+                    see_complete.append(True)
+                    break
+                # print( 'made it to value= %.3f, elapsed time= %.4f sec (count=%i)' % (v, time.time()-t0, count))
+        self.failUnless(len(see_complete) > (len(vals) - 5))
+
     def test_putwait(self):
         write('Put with wait (using real motor!) \n')
         pv = PV(pvnames.motor1)
@@ -120,14 +147,37 @@ class PV_Tests(unittest.TestCase):
                 break
             count = count + 1
             time.sleep(0.02)
-        write('    put_complete=%s (should be True), and count=%i (should be>3)\n' % 
+        write('    put_complete=%s (should be True), and count=%i (should be>3)\n' %
               (pv.put_complete, count))
-        self.failUnless(pv.put_complete)            
+        self.failUnless(pv.put_complete)
         self.failUnless(count > 3)
 
+    def test_get_callback(self):
+        write("Callback test:  changing PV must be updated\n")
+        global NEWVALS
+        mypv = PV(pvnames.updating_pv1)
+        NEWVALS = []
+        def onChanges(pvname=None, value=None, char_value=None, **kw):
+            write( 'PV %s %s, %s Changed!\n' % (pvname, repr(value), char_value))
+            NEWVALS.append( repr(value))
+
+        mypv.add_callback(onChanges)
+        write('Added a callback.  Now wait for changes...\n')
+
+        t0 = time.time()
+        while time.time() - t0 < 3:
+            time.sleep(1.e-4)
+        write('   saw %i changes.\n' % len(NEWVALS))
+        self.failUnless(len(NEWVALS) > 3)
+        mypv.clear_callbacks()
+
+
     def test_subarrays(self):
         write("Subarray test:  dynamic length arrays\n")
         driver = PV(pvnames.subarr_driver)
+        subarr1 = PV(pvnames.subarr1)
+        subarr1.connect()
+
         len_full = 64
         len_sub1 = 16
         full_data = numpy.arange(len_full)/1.0
@@ -135,9 +185,9 @@ class PV_Tests(unittest.TestCase):
         caput("%s.NELM" % pvnames.subarr1, len_sub1)
         caput("%s.INDX" % pvnames.subarr1, 0)
 
-        subarr1 = PV(pvnames.subarr1)
 
-        driver.put(full_data) ;   time.sleep(0.1)
+        driver.put(full_data) ;
+        time.sleep(0.1)
         subval = subarr1.get()
 
         self.assertEqual(len(subval), len_sub1)
@@ -173,38 +223,25 @@ class PV_Tests(unittest.TestCase):
         self.assertEqual(val, 0)
 
 
-    def xtest_DoubleVal(self):
+    def test_DoubleVal(self):
         pvn = pvnames.double_pv
-        chid = ca.create_channel(pvn,connect=True)
-        cdict  = ca.get_ctrlvars(chid)
-        write( 'CA testing CTRL Values for a Double (%s)\n'   % (pvn))
-        self.failUnless('units' in cdict)
-        self.failUnless('precision' in cdict)
+        pv = PV(pvn)
+        pv.get()
+        cdict  = pv.get_ctrlvars()
+        write( 'Testing CTRL Values for a Double (%s)\n'   % (pvn))
         self.failUnless('severity' in cdict)
-       
-        hostname = ca.host_name(chid)
-        self.failUnless(hostname.startswith(pvnames.double_pv_host))
-
-        count = ca.element_count(chid)
-        self.assertEqual(count,1)
-
-        ftype= ca.field_type(chid)
-        self.assertEqual(ftype,ca.dbr.DOUBLE)
-
-        prec = ca.get_precision(chid)
-        self.assertEqual(prec, pvnames.double_pv_prec)
-
-        units= ca.get_ctrlvars(chid)['units']
+        self.failUnless(len(pv.host) > 1)
+        self.assertEqual(pv.count,1)
+        self.assertEqual(pv.precision, pvnames.double_pv_prec)
+        units= ca.BYTES2STR(pv.units)
         self.assertEqual(units, pvnames.double_pv_units)
+        self.failUnless(pv.access.startswith('read'))
 
-        rwacc= ca.access(chid)
-        self.failUnless(rwacc.startswith('read'))
 
-        
-    def xtest_type_converions_2(self):
+    def test_type_converions_2(self):
         write("CA type conversions arrays\n")
         pvlist = (pvnames.char_arr_pv,
-                  pvnames.long_arr_pv,       
+                  pvnames.long_arr_pv,
                   pvnames.double_arr_pv)
         pause_updating()
         chids = []
@@ -220,7 +257,7 @@ class PV_Tests(unittest.TestCase):
             values[name] = ca.get(chid)
         for promotion in ('ctrl', 'time'):
             for chid, pvname in chids:
-                write('=== %s  chid=%s as %s\n' % (ca.name(chid), 
+                write('=== %s  chid=%s as %s\n' % (ca.name(chid),
                                                    repr(chid), promotion))
                 time.sleep(0.01)
                 if promotion == 'ctrl':
@@ -229,7 +266,7 @@ class PV_Tests(unittest.TestCase):
                     ntype = ca.promote_type(chid, use_time=True)
 
                 val  = ca.get(chid, ftype=ntype)
-                cval = ca.get(chid, as_string=True)    
+                cval = ca.get(chid, as_string=True)
                 for a, b in zip(val, values[pvname]):
                     self.assertEqual(a, b)
 
@@ -242,8 +279,6 @@ if __name__ == '__main__':
 
 #     chid = ca.create_channel(pvnames.int_pv,
 #                              callback=onConnect)
-#     
+#
 #     time.sleep(0.1)
-#     print chid, ca.name(chid)
-#     print CONN_DAT
-# ;    
+
