@@ -7,7 +7,6 @@
 # you should provide the host_name, units, and precision.  It
 # is assumed to have count=1
 double_pv = 'Py:ao1'
-double_pv_host = 'vincent'
 double_pv_units = 'microns'
 double_pv_prec = 4
 
@@ -54,6 +53,8 @@ updating_str1 = 'Py:char256'
 updating_pvlist = ['Py:ao1', 'Py:ai1', 'Py:long1', 'Py:ao2']
 #### alarm test
 
+non_updating_pv = 'Py:ao4'
+
 alarm_pv = 'Py:long1'
 alarm_comp='ge'
 alarm_trippoint = 7
