@@ -1,3 +1,4 @@
+from __future__ import print_function
 
 import time
 import threading
@@ -6,9 +7,9 @@ import pvnames
 def threaded_pvput(pv, value):
     "put-with-wait for calling in a thread"
     t0 = time.time()
-    print ' - threaded_pvput starting at ', pv.get()
+    print(' - threaded_pvput starting at ', pv.get())
     pv.put(value, wait=True, timeout=10.0)
-    print ' - threaded_pvput done (%.3f sec)' % (time.time()-t0)
+    print(' - threaded_pvput done (%.3f sec)' % (time.time()-t0))
    
 if __name__ == '__main__':
     pvname = pvnames.motor2
@@ -22,6 +23,6 @@ if __name__ == '__main__':
                           args=(pv, target))
     th.start()
     th.join()
-    print 'All Done.'
+    print('All Done.')
         
  
